package ufrn.imd.controller.impl;

import lombok.extern.java.Log;
import ufrn.imd.controller.Controller;
import ufrn.imd.service.impl.WithdrawalServiceImpl;
import ufrn.imd.utils.ServerResponse;
import ufrn.imd.utils.message.WithdrawalMessage;

import java.net.MalformedURLException;
import java.rmi.AlreadyBoundException;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.server.UnicastRemoteObject;
import java.util.Optional;

import static ufrn.imd.utils.NameSpace.HOST_WITHDRAWAL_SERVICE;
import static ufrn.imd.utils.NameSpace.PORT_WITHDRAWAL_SERVICE;

@Log
public class WithdrawalControllerImpl extends UnicastRemoteObject implements Controller {
    private final WithdrawalServiceImpl service;

    public WithdrawalControllerImpl(WithdrawalServiceImpl service) throws RemoteException, MalformedURLException, AlreadyBoundException {
        super();
        log.info("Starting Withdrawal service!");
        this.service = service;
        log.info(String.format("Initializing server in %s", HOST_WITHDRAWAL_SERVICE));
        LocateRegistry.createRegistry(PORT_WITHDRAWAL_SERVICE);
        Naming.bind(HOST_WITHDRAWAL_SERVICE, this);
    }

    @Override
    public ServerResponse getResponse(Object client) throws RemoteException {
        WithdrawalMessage withdrawalMessage = (WithdrawalMessage) client;
        ServerResponse response;
        try {
            response = this.service.withdraw(withdrawalMessage.getValue(), Optional.of(withdrawalMessage.getClient()));
        } catch (RuntimeException e) {
            e.printStackTrace();
            response = ServerResponse.builder()
                    .status(500)
                    .message(e.getMessage())
                    .body(withdrawalMessage.getClient())
                    .build();
        }
        return response;
    }
}
